function phaseout = GenContinuous_pendulum(input)

% Extract Parameters
ns=input.auxdata.ns;
A=input.auxdata.A;
B=input.auxdata.B;
Atilda=input.auxdata.Atilda;
Ni=input.auxdata.Ni;
Nitilda=input.auxdata.Nitilda;
D1tilda=input.auxdata.D1tilda;
D2tilda=input.auxdata.D2tilda;
Sigmatilda=input.auxdata.Sigmatilda;
Q=input.auxdata.Q;
r=input.auxdata.rescale;
R=input.auxdata.R;

% Variables
t = input.phase.time;
x = input.phase.state;
u = input.phase.control;

N=numel(t);

mdot=zeros(ns,N);
cdot=zeros(size(x,2)-ns,N);
integrand=zeros(N,1);
posdefcov=zeros(N,2);

for i=1:N
    
    m = x(i,1:ns)'; % state (mean part)
    c = x(i,ns+1:end)'; % state (covariance part)
    u_=u(i,:)';
    u1=u(i,1);
    u2=u(i,2);

    % mean and covariance propagation
    mdot(:,i)=A*m+B*u_+(Ni*u1+Ni*u2)*m;
    cdot(:,i)=Atilda*c+(Nitilda*u1+Nitilda*u2)*c+Sigmatilda+...
        D1tilda*kron(u_,u_)+D2tilda*kron(u_,u_);

    Qt=Q';
    integrand(i) = r*(u_'*R*u_ + m'*Q*m + (Qt(:))'*c );

    cmat=reshape(c,[ns ns]);
  
    if max(isnan(cmat(:)))==1
        eigvals=nan(1,ns);
    else
        eigvals=eig(cmat);
    end   
    posdefcov(i,:)=eigvals';
    
end

xdot               = [mdot',cdot'];

phaseout.dynamics  = xdot;
phaseout.integrand = integrand;

phaseout.path=posdefcov;
