%-------------------------------------------------------------------------%
%------------- Provide and Set Up All Bounds for Problem -----------------%
%-------------------------------------------------------------------------%
mmin = -pi/2 ; % limit for pos
mmax = pi/2 ; % limit for pos
mdotmin = -100 ; % limit for vel
mdotmax = 100 ; % limit for vel
cmin = 0 ; % min abs covariance
cmax = 10; % max abs covariance
umin=zeros([1 nu]);
umax=repmat(1,[1 nu])*100;

xmin=[mmin,mdotmin,cmin, -cmax, -cmax, cmin];
xmax=[mmax,mdotmax,repmat(cmax,[1 4])];

t0=0;
x0=[m0' c0'];
x0min=x0;
x0max=x0;

tfmin=tfguess;
tfmax=tfguess;
xfmin=[m0' xmin(3:end)];
xfmax=[m0' xmax(3:end)];

bounds.phase.initialtime.lower  = t0;
bounds.phase.initialtime.upper  = t0;
bounds.phase.finaltime.lower    = tfmin;
bounds.phase.finaltime.upper    = tfmax;
bounds.phase.initialstate.lower = x0min;
bounds.phase.initialstate.upper = x0max;
bounds.phase.state.lower        = xmin;
bounds.phase.state.upper        = xmax;
bounds.phase.finalstate.lower   = xfmin;
bounds.phase.finalstate.upper   = xfmax;
bounds.phase.control.lower      = umin;
bounds.phase.control.upper      = umax;
bounds.phase.integral.lower = 0;
bounds.phase.integral.upper = 1e3;

bounds.phase.path.lower      =  [0 0];
bounds.phase.path.upper      =  [10000 10000];

%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%

% Random guess
% tGuess               = [t0;tfguess];
% xGuess               = [x0;0*x0];
% uGuess               = 10*rand(2);
% guess.phase.time     = tGuess;
% guess.phase.state    = xGuess;
% guess.phase.control  = uGuess;
% guess.phase.integral = 1e2;

% Initial guess
load iniguess.mat
tGuess               = outputlast.result.solution.phase.time;
xGuess               = outputlast.result.solution.phase.state;
tauGuess             = outputlast.result.solution.phase.control+5*rand(size(outputlast.result.solution.phase.control));
guess.phase.time     = tGuess;
guess.phase.state    = xGuess;
guess.phase.control  = tauGuess;
guess.phase.integral = outputlast.result.solution.phase.integral; 

%-------------------------------------------------------------------------%
%----------Provide Mesh Refinement Method and Initial Mesh ---------------%
%-------------------------------------------------------------------------%
mesh.method          = 'hp1';
mesh.tolerance       = 1e-10;
mesh.maxiteration    = 1;
mesh.colpointsmax    = 10;
mesh.colpointsmin    = 2;
N                    = 1; % number of mesh intervals
mesh.phase.colpoints = 50*ones(1,N); % number of collocation pts per interval
mesh.phase.fraction  =  ones(1,N)/N;


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%        
%-------------------------------------------------------------------------%
setup.name                        = 'pendulum';
setup.functions.continuous        = @GenContinuous_pendulum;
setup.functions.endpoint          = @GenEndPoint_pendulum;
setup.auxdata                     = auxdata;
setup.bounds                      = bounds;
setup.guess                       = guess;
setup.nlp.solver                  = 'snopt';
setup.derivatives.derivativelevel = 'first';
setup.method                      = 'RPMintegration';
setup.mesh                        = mesh;

%-------------------------------------------------------------------------%
%---------------------- Solve Problem Using GPOPS2 -----------------------%
%-------------------------------------------------------------------------%
output = gpops2(setup);
outputlast=output;
