function phaseout = GenContinuous_forearmflex(input)

% Extract Parameters
ns=input.auxdata.ns; 
ns2=input.auxdata.ns2 ;

sigma1=input.auxdata.sigma1;
d1=input.auxdata.d1;

Q=input.auxdata.Q;
Qnoise=input.auxdata.Qnoise;
r=input.auxdata.rescale;
R=input.auxdata.R;

g=input.auxdata.g;
b=input.auxdata.b;
ma=input.auxdata.ma;
l=input.auxdata.l;
Ks=input.auxdata.Ks;
Kv=input.auxdata.Kv;
J=input.auxdata.J;
G=input.auxdata.G;

% Variables
t = input.phase.time;
x = input.phase.state;
u = input.phase.control;

N=numel(t);
mdot=zeros(ns,N);
cdot=zeros(ns2,N);
integrand=zeros(N,1);
posdefcov=zeros(N,ns);

for i=1:N
    
    m = x(i,1:ns)'; % state (mean part)
    q=m(1); qdot=m(2); dtheta=m(3); dthetadot=m(4);
    c = x(i,ns+1:end)'; % state (covariance part)
      
    Pupper = triu(ones(ns));
    Pupper(Pupper==1) = c;
    P=(Pupper+Pupper')-bsxfun(@times,eye(size(Pupper)),diag(Pupper));

    u_=u(i,:)';
    u1=u(i,1);
    u2=u(i,2);

    f= [ qdot ; ...
        (G*u1-b*qdot+ma*g*l*sin(q))/J ; ...
        dthetadot ; ...
        (-Ks*u2*dtheta-Kv*sqrt(u2)*dthetadot-b*dthetadot+ma*g*l*(sin(dtheta+q)-sin(q)))/J];
    
    f_x = reshape([0.0,(g*l*ma*cos(q))/J,0.0,(g*l*ma*(cos(dtheta+q)-cos(q)))/J,1.0,-b/J,0.0,0.0,0.0,0.0,0.0,-(Ks*u2-g*l*ma*cos(dtheta+q))/J,0.0,0.0,1.0,-(b+Kv*sqrt(u2))/J],[4,4]);
    
    % mean propagation
    mdot(:,i)=f ;
   
    % covariance propagation
    Gnoise=[0 0 ; 0 0; 0 0 ; sigma1 d1*u1 ]/J; % noise at dthetaddot level   
    Pdot=f_x*P+P*f_x'+Gnoise*Qnoise*Gnoise';
    cdot(:,i)=Pdot(triu(true(size(Pdot))));

    Qel=Q(triu(true(size(Q))));
   
    integrand(i) = r*(u_'*R*u_ + Qel'*c );
    
    if max(isnan(P(:)))==1
        eigvals=nan(1,ns);
    else
        eigvals=eig(P);
    end
        
    posdefcov(i,:)=eigvals';
  
end

xdot               = [mdot',cdot'];

phaseout.dynamics  = xdot;
phaseout.integrand = integrand;

phaseout.path=posdefcov;

