%-------------------------------------------------------------------------%
%------------- Provide and Set Up All Bounds for Problem -----------------%
%-------------------------------------------------------------------------%
mmin = -10 ; % limit for pos
mmax = 10 ; % limit for pos
mdotmin = -100 ; % limit for vel
mdotmax = 100 ; % limit for vel
cmin = 0 ; % min abs covariance 
cmax = 1e5; % max abs covariance 
umin=[-1e1  0 ];
umax=[ 1e1  50];

Pdiagmin=diag(repmat(cmin,[1 ns]));
Ptriumin=triu(ones(ns),1);
Ptriumin(Ptriumin==1)=-cmax*ones(ns2-ns,1);
Pmin=Pdiagmin+(Ptriumin+Ptriumin');
cmin=Pmin(triu(true(size(Pmin))));
xmin=[mmin,mdotmin,mmin,mdotmin,cmin'];

Pdiagmax=diag(repmat(cmax,[1 ns]));
Ptriumax=triu(ones(ns),1);
Ptriumax(Ptriumax==1)=cmax*ones(ns2-ns,1);
Pmax=Pdiagmax+(Ptriumax+Ptriumax');
cmax=Pmax(triu(true(size(Pmax))));
xmax=[mmax,mdotmax,mmax,mdotmax,cmax'];

% initial and final state
t0=0;
x0min=[m0(1:2)' 0 0 0 0 0 0 0 cmin(6:10)']; % free initial and final covariance
x0max=[m0(1:2)' 0 0 0 0 0 0 0 cmax(6:10)'];
xfmin=[mf(1:2)' 0 0 0 0 0 0 0 cmin(6:10)'];
xfmax=[mf(1:2)' 0 0 0 0 0 0 0 cmax(6:10)'];

% duration
tfmin=tfguess;
tfmax=tfguess;

bounds.phase.initialtime.lower  = t0;
bounds.phase.initialtime.upper  = t0;
bounds.phase.finaltime.lower    = tfmin;
bounds.phase.finaltime.upper    = tfmax;
bounds.phase.initialstate.lower = x0min;
bounds.phase.initialstate.upper = x0max;
bounds.phase.state.lower        = xmin;
bounds.phase.state.upper        = xmax;
bounds.phase.finalstate.lower   = xfmin;
bounds.phase.finalstate.upper   = xfmax;
bounds.phase.control.lower      = umin;
bounds.phase.control.upper      = umax;
bounds.phase.integral.lower = 0;
bounds.phase.integral.upper = 1e8;
bounds.eventgroup.lower         = -1e-8*ones(size(cmax')); 
bounds.eventgroup.upper         =  1e-8*ones(size(cmax'));
bounds.phase.path.lower      =  zeros(1,ns);
bounds.phase.path.upper      =  1e5*ones(1,ns);

%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%

% Random guess
% tGuess               = [t0;tfguess];
% xGuess               = [(x0min+x0max)/2;(xfmin+xfmax)/2];
% uGuess               = rand(2,nu)*10;
% guess.phase.time     = tGuess;
% guess.phase.state    = xGuess;
% guess.phase.control  = uGuess;
% guess.phase.integral = 1;

% Initial guess
load iniguess.mat
tGuess               = outputlast.result.solution.phase.time;
xGuess               = outputlast.result.solution.phase.state;
tauGuess             = outputlast.result.solution.phase.control;
guess.phase.time     = tGuess;
guess.phase.state    = [xGuess(:,1:ns) zeros(numel(tGuess),ns2)];
guess.phase.control  = tauGuess+1e-4*rand(size(tauGuess));
guess.phase.integral = outputlast.result.solution.phase.integral; 

%-------------------------------------------------------------------------%
%----------Provide Mesh Refinement Method and Initial Mesh ---------------%
%-------------------------------------------------------------------------%
mesh.method          = 'hp1';
mesh.tolerance       = 1e-14;
mesh.maxiteration    = 1;
mesh.colpointsmax    = 50;
mesh.colpointsmin    = 2;
N                    = 1; % number of mesh intervals
mesh.phase.colpoints = 50*ones(1,N); % number of collocation pts per interval
mesh.phase.fraction  =  ones(1,N)/N;


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%        
%-------------------------------------------------------------------------%
setup.name                        = 'forearmflex';
setup.functions.continuous        = @GenContinuous_forearmflex;
setup.functions.endpoint          = @GenEndPoint_forearmflex;
setup.auxdata                     = auxdata;
setup.bounds                      = bounds;
setup.guess                       = guess;
setup.nlp.solver                  = 'snopt';
setup.derivatives.derivativelevel = 'first';
setup.method                      = 'RPMintegration';
setup.mesh                        = mesh;

%-------------------------------------------------------------------------%
%---------------------- Solve Problem Using GPOPS2 -----------------------%
%-------------------------------------------------------------------------%
output = gpops2(setup);
outputlast=output;