function phaseout = GenContinuous_forearmcocontraction(input)

% Extract Parameters
ns=input.auxdata.ns; 
ns2=input.auxdata.ns2 ;

sigma1=input.auxdata.sigma1;
d1=input.auxdata.d1;

% Cost parameters
Qnoise=input.auxdata.Qnoise;
rglobal=input.auxdata.rescale;
R=input.auxdata.R;

% Musculoskeletal parameters
M=input.auxdata.M;
a1=input.auxdata.a1;
a2=input.auxdata.a2;
k=input.auxdata.k;
k0=input.auxdata.k0;
b=input.auxdata.b;
b0=input.auxdata.b0;
r1=input.auxdata.r1;
r2=input.auxdata.r2;
l0lm1=input.auxdata.l0lm1;
l0lm2=input.auxdata.l0lm2;

% Variables
t = input.phase.time;
x = input.phase.state;
u = input.phase.control;

N=numel(t);
mdot=zeros(ns,N);
cdot=zeros(ns2,N);
integrand=zeros(N,1);
posdefcov=zeros(N,ns);

for i=1:N
    
    m = x(i,1:ns)'; % state (mean part)
    q=m(1); qdot=m(2); 
    c = x(i,ns+1:end)'; % state (covariance part)
      
    Pupper = triu(ones(ns));
    Pupper(Pupper==1) = c;
    P=(Pupper+Pupper')-bsxfun(@times,eye(size(Pupper)),diag(Pupper));

    u_=u(i,:)';
    u1=u(i,1);
    u2=u(i,2);

    % muscle 1
    l1dot=-a1*qdot;
    tens1=(k0+k*u1)*(r1*u1+l0lm1+a1*q)-(b0+b*u1)*l1dot;
    tau1=-a1*tens1;
    % muscle 2
    l2dot=-a2*qdot;
    tens2=(k0+k*u2)*(r2*u2+l0lm2+a2*q)-(b0+b*u2)*l2dot;
    tau2=-a2*tens2;

    f= [ qdot ; ...
        (tau1+tau2)/M ];
    
    f_x = reshape([0.0,-(a1^2*(k0+k*u1)+a2^2*(k0+k*u2))/M,1.0,-(a1^2*(b0+b*u1)+a2^2*(b0+b*u2))/M],[2,2]);
    
    % mean propagation
    mdot(:,i)= f ;
    
    % covariance propagation
    Gnoise=[0 0 ; sigma1 d1*(abs(u1-u2)^1.5 + 0.01*abs(u1+u2)^1.5) ]/M*a1; % noise at thetaddot level
    Pdot=f_x*P + P*f_x' + Gnoise*Qnoise*Gnoise';
 
    cdot(:,i)=Pdot(triu(true(size(Pdot))));
   
    integrand(i) = rglobal*(u_'*R*u_);
    
    if max(isnan(P(:)))==1
        eigvals=nan(1,ns);
    else
        eigvals=eig(P);
    end
    posdefcov(i,:)=eigvals';

end

xdot               = [mdot',cdot'];

phaseout.dynamics  = xdot;
phaseout.integrand = integrand;
phaseout.path=posdefcov;

