%-------------------------------------------------------------------------%
%------------- Provide and Set Up All Bounds for Problem -----------------%
%-------------------------------------------------------------------------%
mmin1 = -pi ; % limit for q1
mmax1 = pi ; % limit for q1
mmin2 = 0 ; % limit for q2
mmax2 = pi ; % limit for q2
mdotmin = -10 ; % limit for vel
mdotmax = 10 ; % limit for vel

cmin = 0 ; % min abs covariance
cmax = 1; % max abs covariance

umin=[0 0 0 0 0 0]; 
umax=[1 1 1 1 1 1]*10;

Pdiagmin=diag(repmat(cmin,[1 ns]));
Ptriumin=triu(ones(ns),1);
Ptriumin(Ptriumin==1)=-cmax*ones(ns2-ns,1);
Pmin=Pdiagmin+Ptriumin+Ptriumin';
cmin=Pmin(triu(true(size(Pmin))));
xmin=[mmin1,mmin2,mdotmin,mdotmin, cmin' ];

Pdiagmax=diag(repmat(cmax,[1 ns]));
Ptriumax=triu(ones(ns),1);
Ptriumax(Ptriumax==1)=cmax*ones(ns2-ns,1);
Pmax=Pdiagmax+Ptriumax+Ptriumax';
cmax=Pmax(triu(true(size(Pmax))));
xmax=[mmax1,mmax2,mdotmax,mdotmax, cmax'];

% initial and final state
t0=0;
x0=[m0' c0'];

x0min=[m0' c0'];
x0max=[m0' c0'];

tfmin=tfguess;
tfmax=tfguess;

xfmin=[mmin1 mmin2 0 0 cmin'];
xfmax=[mmax1 mmax2 0 0 cmax'];

bounds.phase.initialtime.lower  = t0;
bounds.phase.initialtime.upper  = t0;
bounds.phase.finaltime.lower    = tfmin;
bounds.phase.finaltime.upper    = tfmax;
bounds.phase.initialstate.lower = x0min;
bounds.phase.initialstate.upper = x0max;
bounds.phase.state.lower        = xmin;
bounds.phase.state.upper        = xmax;
bounds.phase.finalstate.lower   = xfmin;
bounds.phase.finalstate.upper   = xfmax;
bounds.phase.control.lower      = umin;
bounds.phase.control.upper      = umax;
bounds.phase.integral.lower = 0;
bounds.phase.integral.upper = 1e8;
bounds.eventgroup.lower         = -1e-5; 
bounds.eventgroup.upper         =  1e-5;

bounds.phase.path.lower      =  zeros(1,ns)-1e-6;
bounds.phase.path.upper      =  1e6*ones(1,ns);

%-------------------------------------------------------------------------%
%---------------------- Provide Guess of Solution ------------------------%
%-------------------------------------------------------------------------%
% Random guess 
% tGuess               = [t0;tfguess];
% xGuess               = [x0;(xfmin+xfmax)/2];
% uGuess               = rand(2,nu);
% guess.phase.time     = tGuess;
% guess.phase.state    = xGuess;
% guess.phase.control  = uGuess;
% guess.phase.integral = 1;

% Initial guess
load iniguess_oblique.mat
tGuess               = outputlast.result.solution.phase.time;
xGuess               = outputlast.result.solution.phase.state;
uGuess               = outputlast.result.solution.phase.control;
guess.phase.time     = tGuess;
guess.phase.state    = xGuess;
guess.phase.control  = uGuess+1e-5*rand(size(uGuess));
guess.phase.integral = outputlast.result.solution.phase.integral; 

%-------------------------------------------------------------------------%
%----------Provide Mesh Refinement Method and Initial Mesh ---------------%
%-------------------------------------------------------------------------%
mesh.method          = 'hp1';
mesh.tolerance       = 1e-8;
mesh.maxiteration    = 1;
mesh.colpointsmax    = 10;
mesh.colpointsmin    = 2;
N                    = 1; % number of mesh intervals
mesh.phase.colpoints = 40*ones(1,N); % number of collocation pts per interval
mesh.phase.fraction  =  ones(1,N)/N;


%-------------------------------------------------------------------------%
%------------- Assemble Information into Problem Structure ---------------%        
%-------------------------------------------------------------------------%
setup.name                        = '2-dof';
setup.functions.continuous        = @GenContinuous_nonlinear_2_dof;
setup.functions.endpoint          = @GenEndPoint_nonlinear_2_dof;
setup.auxdata                     = auxdata;
setup.bounds                      = bounds;
setup.guess                       = guess;
setup.nlp.solver                  = 'snopt';
setup.derivatives.derivativelevel = 'first';
setup.method                      = 'RPMintegration';
setup.mesh                        = mesh;

%-------------------------------------------------------------------------%
%---------------------- Solve Problem Using GPOPS2 -----------------------%
%-------------------------------------------------------------------------%
output = gpops2(setup);
outputlast=output;