% Figure 6. Top view of arm trajectories for a pointing-to-a-line experiment.

clear
clc
close all

%-------------------------------------------------------------------------%
%------------- Problem definition
%-------------------------------------------------------------------------%

ns=4; % state dimension [q1,q2,q1dot,q2dot]
ns2=ns*(ns+1)/2; % covariance dimension
nu=6; % control dimension [u1,u2,u3,u4,u5,u6]
nw=2; % noise dimension
auxdata.ns=ns;
auxdata.ns2=ns2;
auxdata.nu=nu;

% segment's length
L1=0.3;
L2=0.35;

% Target line (oblique)
P1=[-.4 .1];
P2=[-0.2 .5];
% Target line (horiz)
% P1=[-0.5 0.55];
% P2=[0.5 .55];

coef=[P1(1) 1 ; P2(1) 1]\[-P1(2); -P2(2)];
aline=coef(1);
bline=1;
cline=coef(2);
auxdata.P1=P1;
auxdata.P2=P2;

% cost parameters
auxdata.coef=1;
weightVariance=1e4; % variance weight
wpos=weightVariance; 
auxdata.Q=auxdata.coef*diag([wpos,wpos,0,0]); 
auxdata.rescale=1; % to rescale the cost globally (only for numerical issues)
auxdata.R=diag(ones(1,nu)); % effort weight 
auxdata.Qf=auxdata.Q;
Qf=auxdata.Qf;
auxdata.weight=1/2; % weight for acceleration cost
auxdata.Qnoise=eye(nw); % noise covariance 

direc = 90*pi/180;
dist = 0.25;
    
% initial mean and covariance
cartini=[0 0.30];
xini0=cartini(1); yini0=cartini(2);
angini=Cart2Ang(cartini,L1,L2);
q10=angini(1); 
q20=angini(2);
m0=[q10 q20 0 0 ]'; 
Pdiag0=diag(zeros(1,ns)); % initial covariance
Ptriu0=triu(ones(ns),1);
Ptriu0(Ptriu0==1)=zeros(ns2-ns,1);
P0=Pdiag0+Ptriu0+Ptriu0';
c0=P0(triu(true(size(P0)))); % initial covariance in vector form

% final state
cartend=[cos(direc); sin(direc)]*dist+cartini';
posxtarg=cartend(1);
auxdata.posxtarg=posxtarg;
posytarg=cartend(2);
auxdata.posytarg=posytarg;
angend=Cart2Ang(cartend',L1,L2);
q1f=angend(1); 
q2f=angend(2);
mf=[q1f q2f 0 0 ]'; 
Pdiagf=diag(zeros(1,ns)); 
Ptriuf=triu(ones(ns),1);
Ptriuf(Ptriuf==1)=zeros(ns2-ns,1);
Pf=Pdiagf+Ptriuf+Ptriuf';
cf=Pf(triu(true(size(Pf)))); % initial covariance in vector form

% initial mean and covariance 
auxdata.m0=m0;
auxdata.c0=c0;
% final mean and covariance (some components may be free)
auxdata.mf=mf;
auxdata.cf=cf;

% movement duration
T=0.55; % seconds
tfguess=T;

% noise properties
% constant noise magnitude
sigma1=1*0.1;
sigma2=sigma1;
% signal dependent noise magnitude
d1=0*0.1;
d2=d1;

auxdata.sigma1=sigma1;
auxdata.sigma2=sigma2;
auxdata.d1=d1;
auxdata.d2=d2;

%-------------------------------------------------------------------------%
%-----------  Find optimal solution (SOOC resolution)
%-------------------------------------------------------------------------%

run GenMain_nonlinear_2_dof

x=output.result.interpsolution.phase(1).state;
u=output.result.interpsolution.phase(1).control;
time=output.result.interpsolution.phase(1).time;
tf=time(end);

% Extract optimal state trajectories and open-loop control
q1opt = x(:,1);
q2opt = x(:,2);
q1dotopt = x(:,3);
q2dotopt = x(:,4); 
u1opt=u(:,1);
u2opt=u(:,2);
u3opt=u(:,3);
u4opt=u(:,4);
u5opt=u(:,5);
u6opt=u(:,6);

% interpolate to create ui(t) functions
u1optpp = interpolate(time,u1opt,'linear','pp'); u1t = @(t) ppval(u1optpp,t);
u2optpp = interpolate(time,u2opt,'linear','pp'); u2t = @(t) ppval(u2optpp,t);
u3optpp = interpolate(time,u3opt,'linear','pp'); u3t = @(t) ppval(u3optpp,t);
u4optpp = interpolate(time,u4opt,'linear','pp'); u4t = @(t) ppval(u4optpp,t);
u5optpp = interpolate(time,u5opt,'linear','pp'); u5t = @(t) ppval(u5optpp,t);
u6optpp = interpolate(time,u6opt,'linear','pp'); u6t = @(t) ppval(u6optpp,t);

%-------------------------------------------------------------------------%
%------------- Simulate trajectories with noise
%-------------------------------------------------------------------------%

nrep=20; % number of sampled trajectories
dt=0.001;
N=fix(tf/dt);

for k=1:nrep
    xrun(:,1)=x(1,1:ns)'; % initial state
    for i=1:N-1
        
        ti=(i-1)*dt;
        q1=xrun(1,i); q2=xrun(2,i); q1dot=xrun(3,i); q2dot=xrun(4,i);   
        u1=u1t(ti); u2=u2t(ti); u3=u3t(ti); u4=u4t(ti); u5=u5t(ti); u6=u6t(ti);   

        q1ddot=(250000*((q1dot*((1081*u1)/10 + 541/10))/625 + (q1dot*((1081*u2)/10 + 541/10))/625 - (567*q2dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/250 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/250 - (((8108*u1)/5 + 4054/5)*((3491*u1)/100000 - q1/25 + 2269/25000))/25 + (((8108*u2)/5 + 4054/5)*(q1/25 + (3491*u2)/100000 + 2793/100000))/25 - (567*q1dot*q2dot*sin(q2))/3125))/(9*(3888*cos(q2)^2 - 4925)) - (50000*(54*cos(q2) + 35)*((q2dot*((1081*u3)/10 + 541/10))/1600 + (q2dot*((1081*u4)/10 + 541/10))/1600 + (567*q1dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/200 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/200 - (((8108*u3)/5 + 4054/5)*((1091*u3)/50000 - q2/40 + 709/12500))/40 + (((8108*u4)/5 + 4054/5)*(q2/40 + (1091*u4)/50000 - 109/25000))/40))/(63*(3888*cos(q2)^2 - 4925)) + 0*((7*cos(q1 + q2))/20 + (3*cos(q1))/10)*((250000*((7*sin(q1 + q2))/20 + (3*sin(q1))/10))/(9*(3888*cos(q2)^2 - 4925)) - (2500*sin(q1 + q2)*(54*cos(q2) + 35))/(9*(3888*cos(q2)^2 - 4925)));
        q2ddot=0*((625*sin(q1 + q2)*(3024*cos(q2) + 3935))/(63*(3888*cos(q2)^2 - 4925)) - (50000*(54*cos(q2) + 35)*((7*sin(q1 + q2))/20 + (3*sin(q1))/10))/(63*(3888*cos(q2)^2 - 4925)))*((7*cos(q1 + q2))/20 + (3*cos(q1))/10) - (50000*(54*cos(q2) + 35)*((q1dot*((1081*u1)/10 + 541/10))/625 + (q1dot*((1081*u2)/10 + 541/10))/625 - (567*q2dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/250 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/250 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/250 - (((8108*u1)/5 + 4054/5)*((3491*u1)/100000 - q1/25 + 2269/25000))/25 + (((8108*u2)/5 + 4054/5)*(q1/25 + (3491*u2)/100000 + 2793/100000))/25 - (567*q1dot*q2dot*sin(q2))/3125))/(63*(3888*cos(q2)^2 - 4925)) + (12500*(3024*cos(q2) + 3935)*((q2dot*((1081*u3)/10 + 541/10))/1600 + (q2dot*((1081*u4)/10 + 541/10))/1600 + (567*q1dot^2*sin(q2))/6250 + (7*((8108*u5)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 - (2749*u5)/50000 - 7147/50000))/200 + (7*((8108*u6)/5 + 4054/5)*((7*q1)/250 + (7*q2)/200 + (2749*u6)/50000 + 1343/100000))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u5)/10 + 541/10))/200 + (7*((7*q1dot)/250 + (7*q2dot)/200)*((1081*u6)/10 + 541/10))/200 - (((8108*u3)/5 + 4054/5)*((1091*u3)/50000 - q2/40 + 709/12500))/40 + (((8108*u4)/5 + 4054/5)*(q2/40 + (1091*u4)/50000 - 109/25000))/40))/(441*(3888*cos(q2)^2 - 4925));
    
        fdyn= [  q1dot; ...
                 q2dot; ...
                 q1ddot; ...
                 q2ddot ];

        Gnoise =[  0, 0 ; ...
                   0, 0 ; ...
                  -(250000*sigma1)/(9*(3888*cos(q2)^2 - 4925)),       (50000*sigma2*(54*cos(q2) + 35))/(63*(3888*cos(q2)^2 - 4925)) ; ...
                  (50000*sigma1*(54*cos(q2) + 35))/(63*(3888*cos(q2)^2 - 4925)), -(12500*sigma2*(3024*cos(q2) + 3935))/(441*(3888*cos(q2)^2 - 4925))];

        xrun(:,i+1)=xrun(:,i)+fdyn*dt + ...
                     Gnoise*sqrtm(auxdata.Qnoise)*randn(nw,1)*sqrt(dt);  
      
    end
     
    q1all(:,k)=xrun(1,:)';
    q2all(:,k)=xrun(2,:)';   
    q1dotall(:,k)=xrun(3,:)'; 
    q2dotall(:,k)=xrun(4,:)'; 
end

time2=linspace(0,tf,N)';

for ii=1:size(q1all,1) 
   % positions
   q1std(ii)=std(q1all(ii,:));
   q1mean(ii)=mean(q1all(ii,:));
   q2std(ii)=std(q2all(ii,:));
   q2mean(ii)=mean(q2all(ii,:));
   % velocities
   q1dotstd(ii)=std(q1dotall(ii,:));
   q1dotmean(ii)=mean(q1dotall(ii,:));
   q2dotstd(ii)=std(q2dotall(ii,:));
   q2dotmean(ii)=mean(q2dotall(ii,:));
end

% x axis and y axis velocities
vx=-(L1*sin(q1opt)+L2*sin(q1opt+q2opt)).*q1dotopt - L2*sin(q1opt+q2opt).*q2dotopt;
vy= (L1*cos(q1opt)+L2*cos(q1opt+q2opt)).*q1dotopt + L2*cos(q1opt+q2opt).*q2dotopt;

posxopt =L1*cos(q1opt)+L2*cos(q1opt+q2opt);
posyopt =L1*sin(q1opt)+L2*sin(q1opt+q2opt);

%-------------------------------------------------------------------------%
%------------- Plot figures
%-------------------------------------------------------------------------%

% Compute cartesian coordinates starting from joints information
fid=figure; hold on;
for ii=1:nrep
 
    posx =L1*cos(q1all(:,ii))+L2*cos(q1all(:,ii)+q2all(:,ii));
    posy =L1*sin(q1all(:,ii))+L2*sin(q1all(:,ii)+q2all(:,ii));
    
    posxendall(ii) =posx(end);
    posyendall(ii) =posy(end);
    
    set(gca,'FontSize',14,'fontName','Times New Roman')
    
    Xp=[posx' NaN];
    Yp=[posy' NaN];
    p = patch(Xp,Yp,'k');
    set(p,'edgecolor','k','linewidth',1,'edgealpha',0.20)
    plot(posx(end),posy(end),'.k');

end

% mean displacements
posx_=L1*cos(q1opt);
posy_=L1*sin(q1opt);
posx =L1*cos(q1opt)+L2*cos(q1opt+q2opt);
posy =L1*sin(q1opt)+L2*sin(q1opt+q2opt);

for ti=1:numel(posx)-1:numel(posx)

    plot([0 posx_(ti)], ... 
         [0 posy_(ti)], ... 
         'color',[0 0 0],'LineWidth',5,'LineStyle','-',...
         'Marker','o',...
         'MarkerEdgeColor',[0 0 0],...
         'MarkerFaceColor',[0 0 0] , ....
         'MarkerSize', 4 );
    plot([posx_(ti) posx(ti)], ... 
         [posy_(ti) posy(ti)], ... 
         'color',[0 0 0],'LineWidth',3,'LineStyle','-',...
         'Marker','o',...
         'MarkerEdgeColor',[0 0 0],...
         'MarkerFaceColor',[0 0 0] , ....
         'MarkerSize', 4 );

end

plot(linspace(-.5,.5,10),-aline*linspace(-.5,.5,10)-cline,'-b','linewidth',2)
hold on;

% confidence empirical from sampling
[xellip2,yellip2]=confidenceEllipse(posxendall',posyendall');
plot(xellip2,yellip2,'-g','linewidth',2);

xlabel('x (m)')
ylabel('y (m)')

ylim([-0.05 .60])
xlim([-0.2 0.41])
axis equal;